using System;
using System.IO;

namespace RSSStoreViewer
{
	/// <summary>
	/// Summary description for FeedItem.
	/// </summary>
	public class FeedItem
	{
        protected String m_streamname;
        protected String m_name;
        protected String m_link;
        protected String m_pubDate;
        protected String m_body;

        protected int m_index;

        public String Name
        {
            get
            {
                return m_name;
            }
        }

        public int Index
        {
            get
            {
                return m_index;
            }
        }

        public String Link
        {
            get
            {
                return m_link;
            }
        }

        public String Date
        {
            get
            {
                return m_pubDate;
            }
        }

        public String Body
        {
            get
            {
                return m_body;
            }
        }

		public FeedItem(String streamname, int index)
		{
            m_streamname = streamname;
            m_index = index;
		}

        public bool Read(byte[] stream)
        {
            bool bXmlParsingStatus = true;

            MemoryStream m = new MemoryStream(stream);

            System.Xml.XmlTextReader reader = null;

            try
            {  
                reader = new System.Xml.XmlTextReader(m);
                reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None;
    
                
                int nElement = -1;

                while ( reader.Read() )
                {
                    switch ( reader.NodeType )
                    {
                        case System.Xml.XmlNodeType.Element:
                        {
                            switch (reader.Name)
                            {
                                case "description" : nElement = 0; break;
                                case "title" : nElement = 1; break;
                                case "link" : nElement = 2; break;
                                case "pubDate" : nElement = 3; break;
                                default : nElement = -1; break;
                            }

                        }break;
                        case System.Xml.XmlNodeType.Text:
                        {
                            switch (nElement)
                            {
                                case 0 : m_body = reader.Value; break;
                                case 1 : m_name = reader.Value; break;
                                case 2 : m_link = reader.Value; break;
                                case 3 : m_pubDate = reader.Value; break;
                            }

                            nElement = -1;
                        }break;

                    }       
                } // end while      
            }
            catch( System.Xml.XmlException)
            {
                //MessageBox.Show("exception : " + e.ToString());
                bXmlParsingStatus = false;
            }
            finally
            {
                if (reader != null) 
                    reader.Close();
            }

            m.Close();

            if (bXmlParsingStatus)
            {
                if (m_name == null)
                {
                    if (m_body != null)
                        m_name = "Item " + m_streamname;
                    else 
                        bXmlParsingStatus = false;
                }
                    
            }

            return bXmlParsingStatus;
        }
	}
}
