using System;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Collections;
using System.Runtime.InteropServices;

namespace RSSStoreViewer
{
	/// <summary>
	/// Summary description for FeedStore.
	/// </summary>
    public class FeedStore
    {


        protected ArrayList m_feeds;
        protected String m_feedStorePath;



        public FeedStore() 
        {
            m_feeds = new ArrayList();
        }

        public String FeedStorePath 
        {
            get {
                return m_feedStorePath;
            }
        }

        public Feed this [int index]
        {
            get 
            {
                if (index < 0 || index >= m_feeds.Count)
                    return null;
                return (Feed) m_feeds[index];
            }
        }

        public int Count()
        {
            return m_feeds.Count;
        }

        public ArrayList Feeds()
        {
            return m_feeds;
        }


        public bool Refresh() 
        {

            System.Text.StringBuilder path = new System.Text.StringBuilder(NativeCalls.MAX_PATH);

            NativeCalls.SHGetFolderPath(IntPtr.Zero, 
                NativeCalls.CSIDL_LOCAL_APPDATA, 
                IntPtr.Zero, 
                NativeCalls.SHGFP_TYPE_CURRENT, 
                path); 
        
            m_feedStorePath = path + @"\Microsoft\Feeds\";
            
            String feedStore = m_feedStorePath + "FeedsStore.feedsdb-ms";

            if (!File.Exists(feedStore))
            {
                MessageBox.Show ("No Windows RSS feed store installed.");
                return false;
            }

            return Read(feedStore);
        }


        public bool Read(String feedstorepath)
        {
            // open the storage
            // enum all streams
            //  retain those which begins with @
            //  read streams into Feed objects
            //  close streams
            // close storage
    
            int nbFailedStreams = 0;
            int nbStreams = 0;

            NativeCalls.IStorage strge = null;
            Int32 hr = NativeCalls.StgOpenStorage(feedstorepath, 
                           (NativeCalls.IStorage)null, 
                           NativeCalls.STGM_READ | NativeCalls.STGM_SHARE_EXCLUSIVE, 
                           IntPtr.Zero,
                           0, 
                           out strge);

            if (strge == null)
                return false;

            NativeCalls.IEnumSTATSTG penum = null;

            strge.EnumElements((uint)0,
                               IntPtr.Zero,
                               (uint)0,
                               out penum);
            if (penum == null)
            {
                Marshal.ReleaseComObject(strge);
                return false;
            }

            penum.Reset();

            uint nb = 0;

            do
            {
                nb = 0;
                STATSTG[] statstg = new STATSTG[1];
                penum.Next(1, statstg, out nb);
                if (nb > 0 &&
                    statstg[0].type == NativeCalls.STGTY_STREAM &&
                    statstg[0].pwcsName.Substring(0,1) == "@")
                {
                    UCOMIStream ppstm = null;

                    strge.OpenStream(statstg[0].pwcsName,
                                     IntPtr.Zero/* reserved1*/,
                                     NativeCalls.STGM_READ | NativeCalls.STGM_SHARE_EXCLUSIVE,
                                     (uint)0/* reserved2*/,
                                     out ppstm);
                    if (ppstm != null)
                    {
                        STATSTG statstgstream;
                        ppstm.Stat(out statstgstream, 1);
                        byte[] buffer = new byte[statstgstream.cbSize];
                        GCHandle handle = GCHandle.Alloc(buffer, GCHandleType.Pinned);
                        ppstm.Read(buffer, (int) statstgstream.cbSize, IntPtr.Zero);

                        Feed f = new Feed(m_feeds.Count);
                        if (!f.Read(buffer))
                            nbFailedStreams++;
                        else
                            m_feeds.Add(f);

                        handle.Free();
                        Marshal.ReleaseComObject(ppstm);
                    }
                    else
                    {
                        nbFailedStreams++;
                    }

                    nbStreams++;
                }
            } while (nb > 0);

            Marshal.ReleaseComObject(penum);

            Marshal.ReleaseComObject(strge);

            if (nbFailedStreams > 0 && nbStreams > 0)
            {
                MessageBox.Show (String.Format("{0} out of {1} feeds could not be read for internal file format error reasons.", 
                    nbFailedStreams,
                    nbStreams));
                return false;
            }

            return true;
        }

    }
}
